"use strict";

var canvasWidth;
var canvasHeight;
var ctx;

var fpsDiv;


var createPoly = false;
var points = [];

var polygons = [];

var selectedPolygon = null;
var prevX, prevY;

$(document).ready(function() {	
    var canvasElement = document.getElementById("mainCanvas");
    canvasWidth = canvasElement.width;
    canvasHeight = canvasElement.height;
    
    setupPage(canvasElement);
    
    ctx = canvasElement.getContext("2d");
    
    fpsDiv = document.getElementById("fps");
    
    startAnimation(tick);
});

function setupPage(canvasElement) {
    var $canvas = $(canvasElement);
    
    $canvas.mousedown( function(event) {
        if(createPoly) {
            points.push([event.offsetX, event.offsetY]);
        } else {
            //Search backwards because the newest polygons are at the end
            for(var i = polygons.length-1; i >= 0; i--) {
                if(isWithinPoints(event.offsetX, event.offsetY, polygons[i].points)){
                    selectedPolygon = polygons[i];
                    prevX = event.offsetX;
                    prevY = event.offsetY;
                    break;
                }
            }
        }
        
        //Prevent the I cursor from happening in the main canvas
        return false;
    }).mousemove( function(event) {
        if(selectedPolygon) {
            var moveX = event.offsetX - prevX;
            var moveY = event.offsetY - prevY;
        
            for(var i = 0; i < selectedPolygon.points.length; i++) {
                selectedPolygon.points[i][0] += moveX;
                selectedPolygon.points[i][1] += moveY;
            }
            
            prevX = event.offsetX;
            prevY = event.offsetY;
        }
    });
    
    $(document).mouseup( function(event) {
        selectedPolygon = null;
    });
    
    $("#polygonBtn").click( function(event) {
        createPoly = !createPoly;
        
        if(createPoly) {
            this.innerHTML = "Finish Polygon";
        } else {
            if(points.length >= 3) {
                polygons.push(new Polygon(points));
            }
            points = [];
            this.innerHTML = "Create Polygon";
        }
    });
};

function isWithinPoints(x, y, points) {
    var odd = false;
    
    for(var i = 0; i < points.length; i++) {
        var one = points[i];
        var two = points[(i+1)%points.length];
        
        //Confirms that y is between y1 and y2
        if((one[1] > y) !== (two[1] > y)) {
            //Since we know y is between y1 and y2, we know the two lines intersect
            //But we are only interested in the ray pointing to the positive x
            //direction. So we check if the intersection is to towards positive x.
            var t1= one[0] - x + (( (one[1]-y) / (one[1] - two[1]) )*(two[0]-one[0]));
            
            if(t1 > 0) {
                odd = !odd;
            }
        }
    }
    
    return odd;
}

function tick(timing) {
    update(timing);
    draw(timing);
    
    if(fpsDiv.innerHTML != timing.fps)
        fpsDiv.innerHTML = timing.fps;
}

function update(timing) {
}

function draw(timing) {
    ctx.clearRect(0, 0, canvasWidth, canvasHeight);
    
    for(var i = 0; i < polygons.length; i++) {
        polygons[i].draw();
    }
    
    if(createPoly) {
        drawNewPoly();
    }
}

function drawNewPoly() {
    ctx.fillStyle = "#007700";
    for(var i = 0; i < points.length; i++) {
        ctx.beginPath();
        ctx.arc(points[i][0],points[i][1],2,0,Math.PI*2,true);
        ctx.fill();
    }
    if(points.length > 1) {
        ctx.beginPath();
        ctx.moveTo(points[0][0],points[0][1]);
    
        for(var i = 1; i < points.length; i++) {
            ctx.lineTo(points[i][0],points[i][1]);
        }
        ctx.stroke();
    }
}