"use strict";

var canvasWidth;
var canvasHeight;
var ctx;

var fpsDiv;


var createPoly = false;
var points = [];

var polygons = [];

var selectedPolygon = null;
var prevX, prevY;


var useMTVs = false;

$(document).ready(function() {	
    var canvasElement = document.getElementById("mainCanvas");
    canvasWidth = canvasElement.width;
    canvasHeight = canvasElement.height;
    
    setupPage(canvasElement);
    
    ctx = canvasElement.getContext("2d");
    
    fpsDiv = document.getElementById("fps");
    
    startAnimation(tick);
});

function setupPage(canvasElement) {
    var $canvas = $(canvasElement);
    
    $canvas.mousedown( function(event) {
        if(createPoly) {
            points.push([event.offsetX, event.offsetY]);
        } else {
            //Search backwards because the newest polygons are at the end
            for(var i = polygons.length-1; i >= 0; i--) {
                if(insidePoints(event.offsetX, event.offsetY, polygons[i].points)){
                    selectedPolygon = polygons[i];
                    prevX = event.offsetX;
                    prevY = event.offsetY;
                    break;
                }
            }
        }
        
        //Prevent the I cursor from happening in the main canvas
        return false;
    }).mousemove( function(event) {
        if(selectedPolygon) {
            var moveX = event.offsetX - prevX;
            var moveY = event.offsetY - prevY;
        
            for(var i = 0; i < selectedPolygon.points.length; i++) {
                selectedPolygon.points[i][0] += moveX;
                selectedPolygon.points[i][1] += moveY;
            }
            
            prevX = event.offsetX;
            prevY = event.offsetY;
        }
    });
    
    $(document).mouseup( function(event) {
        selectedPolygon = null;
    });
    
    $("#polygonBtn").click( function() {
        createPoly = !createPoly;
        
        if(createPoly) {
            this.innerHTML = "Finish Polygon";
        } else {
            if(points.length >= 3) {
                polygons.push(new Polygon(convexHull(points)));
            }
            points = [];
            this.innerHTML = "Create Polygon";
        }
    });
    
    $("#pushAwayBtn").click( function() {
        useMTVs = true;
    });
};

function tick(timing) {
    update(timing);
    draw(timing);
    
    if(fpsDiv.innerHTML != timing.fps)
        fpsDiv.innerHTML = timing.fps;
}

function update(timing) {
    var separate = false;
    if(useMTVs) {
        separate = true;
        useMTVs = false;
    }

    for(var i = 0; i < polygons.length; i++) {
        for(var j = i+1; j < polygons.length; j++) {
            var mtv = intersect(polygons[i].points, polygons[j].points);
            
            if(mtv && mtv[0] !== 0 && mtv[1] !== 0) {
                //console.log(i + " " + j);
                
                if(separate) {
                    polygons[i].translate([mtv[0]/2, mtv[1]/2]);
                    polygons[j].translate([mtv[0]/-2, mtv[1]/-2]);
                }
            }
        }
    }
}

function draw(timing) {
    ctx.clearRect(0, 0, canvasWidth, canvasHeight);
    
    for(var i = 0; i < polygons.length; i++) {
        polygons[i].draw();
    }
    
    if(createPoly) {
        drawNewPoly();
    }
}

function drawNewPoly() {
    if(points.length > 1) {
        ctx.beginPath();
        ctx.moveTo(points[0][0],points[0][1]);
    
        for(var i = 1; i < points.length; i++) {
            ctx.lineTo(points[i][0],points[i][1]);
        }
        ctx.stroke();
    }
    
    ctx.fillStyle = "#007700";
    for(var i = 0; i < points.length; i++) {
        ctx.beginPath();
        ctx.arc(points[i][0],points[i][1],2,0,Math.PI*2,true);
        ctx.fill();
    }
}